@@ -6,4 +6,5 @@ NUMERIC_AGGREGATIONS: Set[str] = AVG_AGGREGATIONS.copy()
 NUMERIC_AGGREGATIONS.update(SUM_AGGREGATIONS)
 MIN_AGGREGATIONS: Set[str] = {"min", "minimum"}
 MAX_AGGREGATIONS: Set[str] = {"max", "maximum"}
-COUNT_AGGREGATIONS = {"count"}
+COUNT_AGGREGATIONS: Set[str] = {"count"}
+COUNT_DISTINCT_AGGREGATIONS: Set[str] = {"countdistinct"}
