@@ -1,5 +1,5 @@
 from datetime import date, datetime
-from typing import Dict, List, Optional, Tuple, Union
+from typing import Dict, List, Mapping, MutableMapping, Optional, Tuple, Union, cast
 
 import ibis
 from ibis.expr.api import NumericColumn
@@ -17,6 +17,7 @@ from sql_to_ibis.exceptions.sql_exception import (
 from sql_to_ibis.parsing.aggregation_aliases import (
     AVG_AGGREGATIONS,
     COUNT_AGGREGATIONS,
+    COUNT_DISTINCT_AGGREGATIONS,
     MAX_AGGREGATIONS,
     MIN_AGGREGATIONS,
     NUMERIC_AGGREGATIONS,
@@ -39,16 +40,17 @@ from sql_to_ibis.sql.sql_value_objects import (
     Aggregate,
     Column,
     CountStar,
-    CrossJoin,
     Date,
     Expression,
     GroupByColumn,
-    JoinBase,
     Literal,
+    NestedCrossJoin,
+    NestedJoinBase,
     Number,
     String,
     Subquery,
     Table,
+    TableOrJoinbase,
     Value,
 )
 
@@ -75,14 +77,17 @@ class TransformerBaseClass(Transformer):
     def __init__(
         self,
         table_name_map: Dict[str, str],
-        table_map: Dict[str, Table],
+        table_map: Mapping[str, TableOrJoinbase],
         column_name_map: Dict[str, Dict[str, str]],
         column_to_table_name: Dict[str, Union[str, AmbiguousColumn]],
         _temp_dataframes_dict=None,
     ):
         super().__init__(visit_tokens=False)
         self._table_name_map = table_name_map
-        self._table_map = table_map
+        self._table_map: MutableMapping[str, Table] = {}
+        for name, table in table_map.items():
+            if isinstance(table, Table):
+                self._table_map[name] = table
         self._column_name_map = column_name_map
         self._column_to_table_name = column_to_table_name
         self._temp_dataframes_dict = _temp_dataframes_dict
@@ -104,8 +109,8 @@ class InternalTransformer(TransformerBaseClass):
 
     def __init__(
         self,
-        tables: List[Table],
-        table_map: Dict[str, Table],
+        tables: List[TableOrJoinbase],
+        table_map: Mapping[str, TableOrJoinbase],
         column_name_map: Dict[str, Dict[str, str]],
         column_to_table_name: Dict[str, Union[str, AmbiguousColumn]],
         table_name_map: Dict[str, str],
@@ -118,7 +123,9 @@ class InternalTransformer(TransformerBaseClass):
             column_to_table_name=column_to_table_name,
         )
         self._tables = tables
-        self._table_names_list = [table.name for table in tables]
+        self._table_names_list = [
+            table.name for table in tables if isinstance(table, Table)
+        ]
         self._column_to_table_name = column_to_table_name.copy()  # This must be
         # copied because when ambiguity is resolved in the following method,
         # we don't want that resolution to carry over to other subqueries
@@ -163,9 +170,9 @@ class InternalTransformer(TransformerBaseClass):
             table = self._column_to_table_name[column]
             if isinstance(table, AmbiguousColumn):
                 present_tables = [
-                    ambig_table
-                    for ambig_table in table.tables
-                    if ambig_table in all_selected_table_names
+                    ambiguous_table
+                    for ambiguous_table in table.tables
+                    if ambiguous_table in all_selected_table_names
                 ]
                 if len(present_tables) == 1:
                     self._column_to_table_name[column] = present_tables[0]
@@ -203,6 +210,8 @@ class InternalTransformer(TransformerBaseClass):
             return ibis_column.min()
         if aggregation in COUNT_AGGREGATIONS:
             return ibis_column.count()
+        if aggregation in COUNT_DISTINCT_AGGREGATIONS:
+            return ibis_column.nunique()
         raise UnsupportedColumnOperation(type(ibis_column), aggregation)
 
     def sql_aggregation(self, agg_parts: list):
@@ -229,14 +238,6 @@ class InternalTransformer(TransformerBaseClass):
             typename=column.typename,
         )
 
-    def mul(self, args: Tuple[int, int]):
-        """
-        Returns the product two numbers
-        """
-        arg1 = args[0]
-        arg2 = args[1]
-        return num_eval(arg1) * num_eval(arg2)
-
     def expression_mul(self, args: Tuple):
         """
         Returns the product of two expressions
@@ -247,14 +248,6 @@ class InternalTransformer(TransformerBaseClass):
         arg2 = args[1]
         return arg1 * arg2
 
-    def add(self, args: Tuple):
-        """
-        Returns the sum two numbers
-        """
-        arg1 = args[0]
-        arg2 = args[1]
-        return num_eval(arg1) + num_eval(arg2)
-
     def expression_add(self, args: Tuple):
         """
         Returns the sum of two expressions
@@ -265,14 +258,6 @@ class InternalTransformer(TransformerBaseClass):
         arg2 = args[1]
         return arg1 + arg2
 
-    def sub(self, args: Tuple):
-        """
-        Returns the difference between two numbers
-        """
-        arg1 = args[0]
-        arg2 = args[1]
-        return num_eval(arg1) - num_eval(arg2)
-
     def expression_sub(self, args: Tuple):
         """
         Returns the difference between two expressions
@@ -283,14 +268,6 @@ class InternalTransformer(TransformerBaseClass):
         arg2 = args[1]
         return arg1 - arg2
 
-    def div(self, args: Tuple):
-        """
-        Returns the division of two numbers
-        """
-        arg1 = args[0]
-        arg2 = args[1]
-        return num_eval(arg1) / num_eval(arg2)
-
     def expression_div(self, args):
         """
         Returns the difference between two expressions
@@ -525,10 +502,10 @@ class InternalTransformer(TransformerBaseClass):
         """
         return AliasExpression(str(name[0]))
 
-    def cross_join_expression(self, cross_join_list: List[CrossJoin]):
+    def cross_join_expression(self, cross_join_list: List[NestedCrossJoin]):
         return cross_join_list[0]
 
-    def from_expression(self, expression: List[Union[Subquery, JoinBase, Table]]):
+    def from_expression(self, expression: List[Union[Subquery, NestedJoinBase, Table]]):
         """
         Return a from sql_object token_or_tree
         :param expression:
@@ -637,6 +614,10 @@ class InternalTransformer(TransformerBaseClass):
         """
         return self.rank(tokens, "dense_rank")
 
+    def coalesce_expression(self, tokens: List[Value]):
+        coalesce_args = [token.value for token in tokens]
+        return Column(ibis.coalesce(*coalesce_args))
+
     def select_expression(
         self, expression_and_alias: Tuple[Value, Optional[AliasExpression]]
     ):
@@ -743,12 +724,12 @@ class InternalTransformer(TransformerBaseClass):
         value = specs[0]
         if isinstance(value, Token) and value.value == "UNBOUNDED":
             return None
-        return value
+        return cast(int, value)
 
-    def frame_preceding(self, preceding_specs: List[Token]):
+    def frame_preceding(self, preceding_specs: List[Union[Token, int]]):
         return Preceding(self.__frame_extract(preceding_specs))
 
-    def frame_following(self, following_specs: List[Token]):
+    def frame_following(self, following_specs: List[Union[Token, int]]):
         return Following(self.__frame_extract(following_specs))
 
     def frame_bound(self, extent_expression_list: List[ExtentExpression]):
@@ -782,8 +763,8 @@ class InternalTransformer(TransformerBaseClass):
 class InternalTransformerWithStarVal(InternalTransformer):
     def __init__(
         self,
-        tables: List[Table],
-        table_map: Dict[str, Table],
+        tables: List[TableOrJoinbase],
+        table_map: Mapping[str, Table],
         column_name_map: Dict[str, Dict[str, str]],
         column_to_table_name: Dict[str, Union[str, AmbiguousColumn]],
         table_name_map: Dict[str, str],
