@@ -1,12 +1,16 @@
+from __future__ import annotations
+
 from dataclasses import InitVar, dataclass
 import re
-from typing import ClassVar, Dict, List, Optional, Union
+from typing import Callable, ClassVar, Dict, List, Optional, Union
 
 import ibis
 from ibis.expr.types import AnyColumn, AnyScalar, TableExpr, ValueExpr
 from lark import Tree
 from pandas import Series
 
+from sql_to_ibis.sql.column_utils import rename_duplicates
+
 
 @dataclass(unsafe_hash=True)
 class Table:
@@ -20,7 +24,7 @@ class Table:
     def get_table_expr(self):
         return self._value
 
-    def get_ibis_columns(self):
+    def get_ibis_columns(self) -> List[AnyColumn]:
         return self._value.get_columns(self.column_names)
 
     def get_alias_else_name(self):
@@ -29,7 +33,7 @@ class Table:
         return self.name
 
     @property
-    def column_names(self):
+    def column_names(self) -> List[str]:
         return self._value.columns
 
 
@@ -100,7 +104,7 @@ class Value:
         Returns the table of the current value
         :return:
         """
-        pass
+        raise NotImplementedError
 
     def get_name(self) -> str:
         """
@@ -391,23 +395,47 @@ class Subquery(Table):
 
 
 @dataclass
-class JoinBase:
+class StrictJoinBase:
     left_table: Table
     right_table: Table
     join_type: str
 
-    def get_tables(self) -> List[Table]:
+
+@dataclass
+class NestedJoinBase:
+    left_table: Table
+    right_table: Table
+    join_type: str
+
+    def get_all_join_columns_handle_duplicates(
+        self, left: Table, right: Table
+    ) -> List[AnyColumn]:
+        left_columns = left.get_ibis_columns()
+        right_columns = right.get_ibis_columns()
+        duplicates = set(left.column_names).intersection(right.column_names)
+        left_columns = rename_duplicates(
+            left, duplicates, self.left_table.name, left_columns
+        )
+        right_columns = rename_duplicates(
+            right, duplicates, self.right_table.name, right_columns
+        )
+        return left_columns + right_columns
+
+    def get_tables(self) -> List[TableOrJoinbase]:
         return [self.left_table, self.right_table]
 
-    def get_table_map(self) -> Dict[str, Table]:
+    def get_table_map(self) -> Dict[str, TableOrJoinbase]:
         return {
             self.left_table.get_alias_else_name(): self.left_table,
             self.right_table.get_alias_else_name(): self.right_table,
         }
 
 
+TableOrJoinbase = Union[Table, NestedJoinBase]
+
+
 @dataclass
-class Join(JoinBase):
+class NestedJoin(NestedJoinBase):
     """
     Wrapper for join related info
     """
@@ -416,5 +444,12 @@ class Join(JoinBase):
 
 
 @dataclass
-class CrossJoin(JoinBase):
+class NestedCrossJoin(NestedJoinBase):
     join_type: str = "cross"
+
+
+@dataclass
+class SQLFunction:
+    args: List[Value]
+    name: str
+    ibisFunction: Callable
